%Written by Romina Kazemivala
%Scattered-power Maps (SPM) Technique
clc;
clear all;
close all;
tic
%-------------------------------------------------------------------------
%%                          INPUT 
%-------------------------------------------------------------------------
%load Parameters
%CO, OUT, and RO Scattering Parameters
%1-3 of appendix C
load S11_CO.mat
load S11_RO.mat
load S11_OUT.mat
load S12_CO.mat
load S12_RO.mat
load S12_OUT.mat
load S21_CO.mat
load S21_RO.mat
load S21_OUT.mat
load S22_CO.mat
load S22_RO.mat
load S22_OUT.mat
S11_CO = permute(S11_CO,[1,2,4,3]);
S11_RO = permute(S11_RO,[1,2,4,3]);
S12_CO = permute(S12_CO,[1,2,4,3]);
S12_RO = permute(S12_RO,[1,2,4,3]);
S21_CO = permute(S21_CO,[1,2,4,3]);
S21_RO = permute(S21_RO,[1,2,4,3]);
S22_CO = permute(S22_CO,[1,2,4,3]);
S22_RO = permute(S22_RO,[1,2,4,3]);

%This uses all datasets
s_CO = {S11_CO;S22_CO;S21_CO;S12_CO};
s_RO = {S11_RO;S22_RO;S21_RO;S12_RO};
s_OUT = {S11_OUT;S22_OUT;S21_OUT;S12_OUT};

%this uses only reflection datasets
% s_CO = {S11_CO;S22_CO};
% s_RO = {S11_RO;S22_RO};
% s_OUT = {S11_OUT;S22_OUT};

%--------------------------------------------------------------------------
%% Initialize Variables

[nx, ny, nz,nf]=size(s_CO{1}); %nx, ny and nz are number of samples along x,y and z; fnum is number of frequency samples
ns = size(s_CO);
frng = 1:nf; %Change this if you want to see less data in the given bandwidth
COsize=10; %in mm
scan_step=10; %spatial sampling step in mm

ROcontr=1; %relative permittivity of background (RO)

COcontr=0.1; %known contrast of scattering probe in CO (i.e., the difference between 
% the relative permittivity of the scattering probe and that of the RO)


% Choose type of forward model: Born or Rytov
approx=1; % Born is 1, Rytov is 0. 

scaling_factor=(COsize^2)/(scan_step^2); %calculating scaling factor for the cubic scattering probe. See Eq.(4.102)
co_max=zeros(1,nf);

%Activate frequency scaling
freq_scaling=1;

%--------------------------------------------------------------------------

%% Fill out s_RO if it contains only 1 value (i.e. size(s_RO{1} returns (1,1,nz,nf))

if size(s_RO{1}(:,1,1,1)) == 1
    for s = 1:ns
        for f = 1:nf
        s_RO_new{s}(:,:,f) =  (squeeze(s_RO{s}(1,1,1,f))*ones(nx,ny));
        end
    end
end
s_RO = s_RO_new;

%==========================================================================
%% Obtain real domain datasets using either Born or Rytov's approximation
%==========================================================================
for s = 1:ns
    
    %Born's Approximation: 
    %Subtract RO from CO, OUT
    if approx
        %Extract  Scattered  Portion  of  OUT  Data
        s_OUT_scat{s} = s_OUT{s} - s_RO{s}; 
        for z = 1:nz
            %Extract  Scattered  Portion  of  PSF  Data
            s_CO_scat{s}(:,:,z,:) = squeeze(s_CO{s}(:,:,z,:)) - s_RO{s}; 
        end
    %Rytovs Approximation: 
    %Scattered Field OUT or CO = s_RO * ln (s_(OUT or CO)/s_RO)
    else
        %Extract  Scattered  Portion  of  OUT  Data
        s_OUT_scat{s} = s_RO{s}.*log(s_OUT{s}./s_RO{s});
        for z = 1:nz
                    %Extract  Scattered  Portion  of  PSF  Data
            s_CO_scat{s}(:,:,z,:) = s_RO{s}.*log(squeeze(s_CO{s}(:,:,:,z))./s_RO{s});
        end
    end
    
%==========================================================================    
                   %% Transform to Fourier domain
%==========================================================================    

    for f = frng
        
        
        if freq_scaling %If the frequency normalization is applied on the extracted data
        %prior to the Fourier transform
        maxi = max(max(max(abs(s_CO{s}(:,:,:,f)))));
        else
            maxi=1;
        end 
        s_OUT_fft{s}(:,:,f) = fft2(s_OUT_scat{s}(:,:,f)/maxi);

        for z = 1:nz
            s_CO_fft{s}(:,:,z,f) = fft2(s_CO_scat{s}(:,:,z,f)/maxi);

        end
    end
end
%--------------------------------------------------------------------------
%%                   Scattering Probe (SP) Power Maps
%--------------------------------------------------------------------------
 % line (9) in Image Reconstruction section of Appendix C

% The CO Maps need to be Nz by Nz at every kx, ky position. Structing the
% loops to achieve this

%First we sum across all S-parameters, performing the conjugate
%multiplication
M_CO_nonorm = zeros(nx,ny,nz,nz,nf);

%for CO we need to loop through z twice, once for the plane being measured (zp),
%and once for the PSF plane using to try and measure that particular plane
%(zn)
for zp = 1:nz
    for zn = 1:nz
        for f=frng
            % sum the conjugates across all S-parameters
           for s = 1:ns
              %See Equation (41) on Page 16 of the Report
               M_CO_nonorm(:,:,zp,zn,f) = M_CO_nonorm(:,:,zp,zn,f) + ...
                   s_CO_fft{s}(:,:,zn,f).*conj(s_CO_fft{s}(:,:,zp,f)); 

           end
        end
    end
end
M_CO = sum(M_CO_nonorm,5)/nf;


%--------------------------------------------------------------------------
%%                     Generate OUT Power Maps
%--------------------------------------------------------------------------

%The OUT Maps need to be Nz by 1 at every kx, ky position. Structing the
%loops to achieve this
M_OUT_nonorm = zeros(nx,ny,nz,nf);
M_OUT_norm = zeros(nx,ny,nz);

%OUT only has one plane being measured, so we loop through all PSFs only
for zn = 1:nz
    %we need to normalize at every frequency
    for f=frng
        % sum the conjugates across all S-parameters
        for s = 1:ns
           
            %See Equation (40) on Page 16 of the Report

            M_OUT_nonorm(:,:,zn,f) = M_OUT_nonorm(:,:,zn,f) + ...
                s_OUT_fft{s}(:,:,f).*conj(s_CO_fft{s}(:,:,zn,f));
            
        end
                M_OUT_norm(:,:,zn) = M_OUT_norm(:,:,zn)+ M_OUT_nonorm(:,:,zn,f);

    end
end

% After all the summation we divide by the number of frequencies
M_OUT = M_OUT_norm/nf;

%--------------------------------------------------------------------------
%%                       INVERSE PROBLEM
%--------------------------------------------------------------------------

%Solving a linear system of equations in Equation (42) on Page 16
contrast_func_fft = zeros(nx,ny,nz);
for kx=1:nx
    for ky=1:ny
        %Quantitative Contrast Reconstruction in Fourier Space Step 11 in
        %Apendix C
        contrast_func_fft(kx,ky,:)=pinv(squeeze(M_CO(kx,ky,:,:)))*squeeze(M_OUT(kx,ky,:)); 
    end
end

%Once we have the contrast function (in fourier domain) we can apply our
%multiplication factors and generate the permittivity map (eps_out)
eps_out=zeros(nx,ny,nz); 
for z=1:nz
    %Obtain  Real-space  Contrast  via  2D  Inverse  Fourier  Transform
    %Step 13 in Appendix C
    eps_out(:,:,z)=scaling_factor.*COcontr.*ifftshift(ifft2(contrast_func_fft(:,:,z)))+ROcontr; 
end



%--------------------------------------------------------------------------
%%                      Plotting the results 
%--------------------------------------------------------------------------
%Qualitative image of the F shape data
        drawmap=ifftshift(ifft2(M_OUT(:,:,2)));
        
        V=abs(drawmap);
        figure;
        imagesc(-(nx-1)*scan_step/20:(nx-1)*scan_step/20, -(nx-1)*scan_step/20:(nx-1)*scan_step/20,rot90(V));
        xlabel('x (cm)')
        ylabel('y (cm)')
        title('M_{OUT} in the Z=2 (cm) plane')

int=0; %interpolation yes 1, no 0. Note that the colormap values may change during certain interpolation settings.
int_rate=0.2; %set interpolation rate

if nz == 1 %plotting 2D images (this section can be also run to get the 3D images slice by slice separately: that is why loop over range locations is still there)
    
    for t=1:nz
        figure('Units', 'centimeters', 'OuterPosition',[0 10 70 14]);
        
        subplot(1, 3, 1) %qualitative outcome
        drawmap=ifftshift(ifft2(rot90(M_OUT(:,:,t))));
        
        V=abs(drawmap/max(max(drawmap))); %normalized modulus
        imagesc(-(nx-1)*scan_step/20:(nx-1)*scan_step/20, -(nx-1)*scan_step/20:(nx-1)*scan_step/20,V);
        colorbar
        colormap parula
        shading interp
        alpha(0.9);
        
        subplot(1, 3, 2) %real part of the relative dielectric permittivity distribution
        drawmap=eps_out(:,:,t);
        if int == 1
            [X,Y]=meshgrid(1:nx,1:ny);
            [XI,YI]=meshgrid(1:int_rate:nx,1:int_rate:ny);
            V2=interp2(X,Y,drawmap,XI,YI,'spline');
            imagesc(-10:10, -10:(ny-1)*scan_step/10,real(V2));
        else
            V=real(drawmap);
            imagesc(0:(nx-1)*scan_step/10, 0:(ny-1)*scan_step/10,V);
        end
        colorbar
        colormap parula
        shading interp
        alpha(0.9);
        
        subplot(1, 3, 3) %imaginary part of the relative dielectric permittivity distribution
        if int == 1
            [X,Y]=meshgrid(1:nx,1:ny);
            [XI,YI]=meshgrid(1:int_rate:nx,1:int_rate:ny);
            V2=interp2(X,Y,drawmap,XI,YI,'spline');
            imagesc(0:(nx-1)*scan_step/10, 0:(ny-1)*scan_step/10,imag(V2));
        else
            V=imag(drawmap);
            imagesc(-10:10, -10:(ny-1)*scan_step/10,V);
        end
        colorbar
        colormap parula
        shading interp
        alpha(0.9);
        
        set(gcf, 'color','w');
    end
    
else % plotting 3D images
    
    if int == 1
        Z=ones(nx,ny);
        [X,Y]=meshgrid(1:nx,1:ny);
        [X1,Y1]=meshgrid(linspace(0,(nx-1)*scan_step,nx),linspace(0,(ny-1)*scan_step,ny));
        
        figure
        for k=1:nz
            [XI,YI]=meshgrid(1:int_rate:nx,1:int_rate:ny);
            V3(:,:,k)=interp2(X,Y,eps_out(:, :, k), XI,YI);
            surface(X1,Y1,Z.*k,real(V3(:,:,k)),'FaceColor','texturemap','CDataMapping','scaled')
        end
        colormap parula
        view(3), box on, axis tight square
        shading flat
        set(gca,'YDir','reverse','ZLim',[1 nz])
        colorbar
        
        figure
        for k=1:nz
            [XI,YI]=meshgrid(1:int_rate:nx,1:int_rate:ny);
            V3(:,:,k)=interp2(X,Y,eps_out(:, :, k), XI,YI);
            surface(X1,Y1,Z.*k,imag(V3(:,:,k)),'FaceColor','texturemap','CDataMapping','scaled')
        end
        colormap parula
        view(3), box on, axis tight square
        shading flat
        set(gca,'YDir','reverse','ZLim',[1 nz])
        colorbar
        
    else 
        Z=ones(nx,ny);
        [X,Y]=meshgrid(-10:10,-10:10);
        
        
        
                %Plot the real part of the permittivity

        figure
        for k=1:nz
            surface(X,Y,Z.*k,real(eps_out(:,:,k)),'FaceColor','texturemap','CDataMapping','scaled')
        end
        title('Real')
xlabel('x (cm)');
ylabel('y (cm)');
zlabel('z (cm)');
axis tight
view(30,40)
grid on
xh = get(gca,'XLabel'); % Handle of the x label
set(xh, 'Units', 'Normalized')
pos = get(xh, 'Position');
set(xh,'Rotation',25)
yh = get(gca,'YLabel'); % Handle of the y label
set(yh, 'Units', 'Normalized')
pos = get(yh, 'Position');
set(yh,'Rotation',-40)
        colormap parula
        view(3), box on, axis tight square
        shading flat
        set(gca,'YDir','reverse','ZLim',[1 nz])
        colorbar
        
        
        %Plot the imaginary part of the permittivity
        figure
        for k=1:nz
            surface(X,Y,Z.*k,imag(eps_out(:,:,k)),'FaceColor','texturemap','CDataMapping','scaled')
        end
        title('Imaginary')
        xlabel('x (cm)');
        ylabel('y (cm)');
        zlabel('z (cm)');
        axis tight
        view(30,40)
        grid on
xh = get(gca,'XLabel'); % Handle of the x label
set(xh, 'Units', 'Normalized')
pos = get(xh, 'Position');
set(xh,'Rotation',25)
yh = get(gca,'YLabel'); % Handle of the y label
set(yh, 'Units', 'Normalized')
pos = get(yh, 'Position');
set(yh,'Rotation',-40)
        colormap parula
        view(3), box on, axis tight square
        shading flat
        set(gca,'YDir','reverse','ZLim',[1 nz])
        colorbar
                caxis([-0.04 0.04])

    end
 end

 toc
